@@ -37,7 +37,7 @@
 #include "knot/common/fdset.h"
 #include "knot/common/log.h"
 #include "knot/nameserver/process_query.h"
-#include "libknot/processing/overlay.h"
+#include "knot/query/layer.h"
 #include "contrib/macros.h"
 #include "contrib/mempattern.h"
 #include "contrib/net.h"
@@ -47,7 +47,7 @@
 
 /*! \brief TCP context data. */
 typedef struct tcp_context {
-	struct knot_overlay overlay;/*!< Query processing overlay. */
+	knot_layer_t layer;         /*!< Query processing layer. */
 	server_t *server;           /*!< Name server structure. */
 	struct iovec iov[2];        /*!< TX/RX buffers. */
 	unsigned client_threshold;  /*!< Index of first TCP client. */
@@ -80,7 +80,7 @@ static enum fdset_sweep_state tcp_sweep(fdset_t *set, int i, void *data)
 	socklen_t len = sizeof(struct sockaddr_storage);
 	if (getpeername(fd, (struct sockaddr*)&ss, &len) == 0) {
 		char addr_str[SOCKADDR_STRLEN] = {0};
-		sockaddr_tostr(addr_str, sizeof(addr_str), &ss);
+		sockaddr_tostr(addr_str, sizeof(addr_str), (struct sockaddr *)&ss);
 		log_notice("TCP, terminated inactive client, address '%s'", addr_str);
 	}
 
@@ -114,8 +114,7 @@ static int tcp_handle(tcp_context_t *tcp, int fd,
 
 	/* Timeout. */
 	rcu_read_lock();
-	conf_val_t *val = &conf()->cache.srv_tcp_reply_timeout;
-	int timeout = conf_int(val) * 1000;
+	int timeout = 1000 * conf()->cache.srv_tcp_reply_timeout;
 	rcu_read_unlock();
 
 	/* Receive data. */
@@ -123,7 +122,7 @@ static int tcp_handle(tcp_context_t *tcp, int fd,
 	if (ret <= 0) {
 		if (ret == KNOT_EAGAIN) {
 			char addr_str[SOCKADDR_STRLEN] = {0};
-			sockaddr_tostr(addr_str, sizeof(addr_str), &ss);
+			sockaddr_tostr(addr_str, sizeof(addr_str), (struct sockaddr *)&ss);
 			log_warning("TCP, connection timed out, address '%s'",
 			            addr_str);
 		}
@@ -132,30 +131,22 @@ static int tcp_handle(tcp_context_t *tcp, int fd,
 		rx->iov_len = ret;
 	}
 
-	knot_mm_t *mm = tcp->overlay.mm;
+	/* Initialize processing layer. */
 
-	/* Initialize processing overlay. */
-	ret = knot_overlay_init(&tcp->overlay, mm);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-	ret = knot_overlay_add(&tcp->overlay, NS_PROC_QUERY, &param);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
+	tcp->layer.state = knot_layer_begin(&tcp->layer, &param);
 
 	/* Create packets. */
-	knot_pkt_t *ans = knot_pkt_new(tx->iov_base, tx->iov_len, mm);
-	knot_pkt_t *query = knot_pkt_new(rx->iov_base, rx->iov_len, mm);
+	knot_pkt_t *ans = knot_pkt_new(tx->iov_base, tx->iov_len, tcp->layer.mm);
+	knot_pkt_t *query = knot_pkt_new(rx->iov_base, rx->iov_len, tcp->layer.mm);
 
 	/* Input packet. */
 	(void) knot_pkt_parse(query, 0);
-	int state = knot_overlay_consume(&tcp->overlay, query);
+	int state = knot_layer_consume(&tcp->layer, query);
 
 	/* Resolve until NOOP or finished. */
 	ret = KNOT_EOK;
 	while (state & (KNOT_STATE_PRODUCE|KNOT_STATE_FAIL)) {
-		state = knot_overlay_produce(&tcp->overlay, ans);
+		state = knot_layer_produce(&tcp->layer, ans);
 
 		/* Send, if response generation passed and wasn't ignored. */
 		if (ans->size > 0 && !(state & (KNOT_STATE_FAIL|KNOT_STATE_NOOP))) {
@@ -167,8 +158,7 @@ static int tcp_handle(tcp_context_t *tcp, int fd,
 	}
 
 	/* Reset after processing. */
-	knot_overlay_finish(&tcp->overlay);
-	knot_overlay_deinit(&tcp->overlay);
+	knot_layer_finish(&tcp->layer);
 
 	/* Cleanup. */
 	knot_pkt_free(&query);
@@ -187,8 +177,7 @@ int tcp_accept(int fd)
 #ifdef SO_RCVTIMEO
 		struct timeval tv;
 		rcu_read_lock();
-		conf_val_t *val = &conf()->cache.srv_tcp_idle_timeout;
-		tv.tv_sec = conf_int(val);
+		tv.tv_sec = conf()->cache.srv_tcp_idle_timeout;
 		rcu_read_unlock();
 		tv.tv_usec = 0;
 		if (setsockopt(incoming, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)) < 0) {
@@ -216,8 +205,8 @@ static int tcp_event_accept(tcp_context_t *tcp, unsigned i)
 
 		/* Update watchdog timer. */
 		rcu_read_lock();
-		conf_val_t *val = &conf()->cache.srv_tcp_hshake_timeout;
-		fdset_set_watchdog(&tcp->set, next_id, conf_int(val));
+		int timeout = conf()->cache.srv_tcp_hshake_timeout;
+		fdset_set_watchdog(&tcp->set, next_id, timeout);
 		rcu_read_unlock();
 
 		return KNOT_EOK;
@@ -232,13 +221,13 @@ static int tcp_event_serve(tcp_context_t *tcp, unsigned i)
 	int ret = tcp_handle(tcp, fd, &tcp->iov[0], &tcp->iov[1]);
 
 	/* Flush per-query memory. */
-	mp_flush(tcp->overlay.mm->ctx);
+	mp_flush(tcp->layer.mm->ctx);
 
 	if (ret == KNOT_EOK) {
 		/* Update socket activity timer. */
 		rcu_read_lock();
-		conf_val_t *val = &conf()->cache.srv_tcp_idle_timeout;
-		fdset_set_watchdog(&tcp->set, i, conf_int(val));
+		int timeout = conf()->cache.srv_tcp_idle_timeout;
+		fdset_set_watchdog(&tcp->set, i, timeout);
 		rcu_read_unlock();
 	}
 
@@ -257,8 +246,8 @@ static int tcp_wait_for_events(tcp_context_t *tcp)
 	if (!is_throttled) {
 		/* Configuration limit, infer maximal pool size. */
 		rcu_read_lock();
-		conf_val_t *val = &conf()->cache.srv_max_tcp_clients;
-		unsigned max_per_set = MAX(conf_int(val) / conf_tcp_threads(conf()), 1);
+		int clients = conf()->cache.srv_max_tcp_clients;
+		unsigned max_per_set = MAX(clients / conf_tcp_threads(conf()), 1);
 		rcu_read_unlock();
 		/* Subtract master sockets check limits. */
 		is_throttled = (set->n - tcp->client_threshold) >= max_per_set;
@@ -315,13 +304,13 @@ int tcp_master(dthread_t *thread)
 	memset(&tcp, 0, sizeof(tcp_context_t));
 
 	/* Create big enough memory cushion. */
-	knot_mm_t mm;
+	knot_mm_t mm = { 0 };
 	mm_ctx_mempool(&mm, 16 * MM_DEFAULT_BLKSIZE);
 
 	/* Create TCP answering context. */
 	tcp.server = handler->server;
 	tcp.thread_id = handler->thread_id[dt_get_id(thread)];
-	tcp.overlay.mm = &mm;
+	knot_layer_init(&tcp.layer, &mm, process_query_layer());
 
 	/* Prepare structures for bound sockets. */
 	conf_val_t val = conf_get(conf(), C_SRV, C_LISTEN);
