@@ -66,6 +66,7 @@ enum knot_error {
 	KNOT_EZONENOENT,
 	KNOT_ENOZONE,
 	KNOT_ENONODE,
+	KNOT_ENORECORD,
 	KNOT_ENOMASTER,
 	KNOT_EDNAMEPTR,
 	KNOT_EPAYLOAD,
@@ -83,6 +84,7 @@ enum knot_error {
 	KNOT_EDSDIGESTLEN,
 	KNOT_ENOTSIG,
 	KNOT_ELIMIT,
+	KNOT_EZONESIZE,
 	KNOT_EWRITABLE,
 	KNOT_EOF,
 	KNOT_ESYSTEM,
@@ -90,8 +92,6 @@ enum knot_error {
 
 	/* Control states. */
 	KNOT_CTL_ESTOP,
-	KNOT_CTL_EACCEPTED,
-	KNOT_CTL_EARG_REQ,
 
 	/* Network errors. */
 	KNOT_NET_EADDR,
@@ -137,9 +137,10 @@ enum knot_error {
 	KNOT_CONF_ENOTINIT,
 	KNOT_CONF_EVERSION,
 	KNOT_CONF_EREDEFINE,
-	KNOT_CONF_ETXN,
-	KNOT_CONF_ENOTXN,
-	KNOT_CONF_EMANYTXN,
+
+	/* Transaction errors. */
+	KNOT_TXN_EEXISTS,
+	KNOT_TXN_ENOTEXISTS,
 
 	/* Processing error. */
 	KNOT_LAYER_ERROR,
