@@ -194,4 +194,11 @@ bool knot_rrset_is_nsec3rel(const knot_rrset_t *rr);
  */
 int knot_rrset_rr_to_canonical(knot_rrset_t *rrset);
 
+/*!
+ * \brief Size of rrset in wire format.
+ *
+ * \retval size in bytes
+ */
+size_t knot_rrset_size(const knot_rrset_t *rrset);
+
 /*! @} */
