@@ -1,4 +1,4 @@
-/*  Copyright (C) 2011 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2016 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -262,6 +262,9 @@ static void print_section_opt(const knot_rrset_t *rr, const uint8_t rcode)
 			printf(";; CLIENT-SUBNET: ");
 			print_edns_client_subnet(opt_data, opt_len);
 			break;
+		case KNOT_EDNS_OPTION_PADDING:
+			printf(";; PADDING: %u B\n", opt_len);
+			break;
 		default:
 			printf(";; Option (%u): ", opt_code);
 			short_hex_print(opt_data, opt_len);
@@ -462,7 +465,7 @@ static void print_error_host(const uint16_t   code,
 	free(owner);
 }
 
-knot_pkt_t* create_empty_packet(const size_t max_size)
+knot_pkt_t *create_empty_packet(const uint16_t max_size)
 {
 	// Create packet skeleton.
 	knot_pkt_t *packet = knot_pkt_new(NULL, max_size, NULL);
@@ -594,6 +597,7 @@ void print_packet(const knot_pkt_t *packet,
 
 	// Print packet information header.
 	if (style->show_header) {
+		print_tls(&net->tls);
 		print_header(packet, style, rcode);
 	}
 
