@@ -27,6 +27,7 @@
 #include "knot/zone/zone-load.h"
 #include "contrib/macros.h"
 #include "contrib/string.h"
+#include "contrib/openbsd/strlcat.h"
 #include "utils/knotc/commands.h"
 #include "utils/knotc/estimator.h"
 
@@ -45,6 +46,15 @@
 #define CMD_ZONE_FLUSH		"zone-flush"
 #define CMD_ZONE_SIGN		"zone-sign"
 
+#define CMD_ZONE_READ		"zone-read"
+#define CMD_ZONE_BEGIN		"zone-begin"
+#define CMD_ZONE_COMMIT		"zone-commit"
+#define CMD_ZONE_ABORT		"zone-abort"
+#define CMD_ZONE_DIFF		"zone-diff"
+#define CMD_ZONE_GET		"zone-get"
+#define CMD_ZONE_SET		"zone-set"
+#define CMD_ZONE_UNSET		"zone-unset"
+
 #define CMD_CONF_INIT		"conf-init"
 #define CMD_CONF_CHECK		"conf-check"
 #define CMD_CONF_IMPORT		"conf-import"
@@ -59,36 +69,46 @@
 #define CMD_CONF_SET		"conf-set"
 #define CMD_CONF_UNSET		"conf-unset"
 
-static int check_args(cmd_args_t *args, unsigned count)
+#define CTL_LOG_STR		"failed to control"
+
+static int check_args(cmd_args_t *args, int min, int max)
 {
-	if (args->argc == count) {
-		return KNOT_EOK;
+	if (max == 0 && args->argc > 0) {
+		log_error("command doesn't take arguments");
+		return KNOT_EINVAL;
+	} else if (min == max && args->argc != min) {
+		log_error("command requires %i arguments", min);
+		return KNOT_EINVAL;
+	} else if (args->argc < min) {
+		log_error("command requires at least %i arguments", min);
+		return KNOT_EINVAL;
+	} else if (max > 0 && args->argc > max) {
+		log_error("command takes at most %i arguments", max);
+		return KNOT_EINVAL;
 	}
 
-	log_error("command requires %u arguments", count);
-
-	return KNOT_EINVAL;
+	return KNOT_EOK;
 }
 
 static int check_conf_args(cmd_args_t *args)
 {
 	// Mask relevant flags.
-	cmd_conf_flag_t flags = args->desc->flags;
-	flags &= CMD_CONF_FOPT_ITEM | CMD_CONF_FREQ_ITEM | CMD_CONF_FOPT_DATA;
+	cmd_flag_t flags = args->desc->flags;
+	flags &= CMD_FOPT_ITEM | CMD_FREQ_ITEM | CMD_FOPT_DATA;
 
 	switch (args->argc) {
 	case 0:
-		if (flags == CMD_CONF_FNONE || (flags & CMD_CONF_FOPT_ITEM)) {
+		if (flags == CMD_FNONE || (flags & CMD_FOPT_ITEM)) {
 			return KNOT_EOK;
 		}
 		break;
 	case 1:
-		if (flags & (CMD_CONF_FOPT_ITEM | CMD_CONF_FREQ_ITEM)) {
+		if (flags & (CMD_FOPT_ITEM | CMD_FREQ_ITEM)) {
 			return KNOT_EOK;
 		}
 		break;
 	default:
-		if (flags != CMD_CONF_FNONE) {
+		if (flags != CMD_FNONE) {
 			return KNOT_EOK;
 		}
 		break;
@@ -157,24 +177,34 @@ static int get_conf_key(const char *key, knot_ctl_data_t *data)
 	return KNOT_EOK;
 }
 
-static void format_data(ctl_cmd_t cmd, knot_ctl_type_t type, knot_ctl_data_t *data,
-                        bool *empty)
+static void format_data(ctl_cmd_t cmd, knot_ctl_type_t data_type,
+                        knot_ctl_data_t *data, bool *empty)
 {
 	const char *error = (*data)[KNOT_CTL_IDX_ERROR];
+	const char *flags = (*data)[KNOT_CTL_IDX_FLAGS];
 	const char *key0  = (*data)[KNOT_CTL_IDX_SECTION];
 	const char *key1  = (*data)[KNOT_CTL_IDX_ITEM];
 	const char *id    = (*data)[KNOT_CTL_IDX_ID];
 	const char *zone  = (*data)[KNOT_CTL_IDX_ZONE];
-	const char *param = (*data)[KNOT_CTL_IDX_TYPE];
+	const char *owner = (*data)[KNOT_CTL_IDX_OWNER];
+	const char *ttl   = (*data)[KNOT_CTL_IDX_TTL];
+	const char *type  = (*data)[KNOT_CTL_IDX_TYPE];
 	const char *value = (*data)[KNOT_CTL_IDX_DATA];
 
+	const char *sign = NULL;
+	if (ctl_has_flag(flags, CTL_FLAG_ADD)) {
+		sign = CTL_FLAG_ADD;
+	} else if (ctl_has_flag(flags, CTL_FLAG_REM)) {
+		sign = CTL_FLAG_REM;
+	}
+
 	switch (cmd) {
 	case CTL_STATUS:
 	case CTL_STOP:
 	case CTL_RELOAD:
 	case CTL_CONF_BEGIN:
-	case CTL_CONF_ABORT:
 	case CTL_CONF_COMMIT:
+	case CTL_CONF_ABORT:
 		// Only error message is expected here.
 		if (error != NULL) {
 			printf("error: (%s)", error);
@@ -186,7 +216,10 @@ static void format_data(ctl_cmd_t cmd, knot_ctl_type_t type, knot_ctl_data_t *da
 	case CTL_ZONE_RETRANSFER:
 	case CTL_ZONE_FLUSH:
 	case CTL_ZONE_SIGN:
-		if (type == KNOT_CTL_TYPE_DATA) {
+	case CTL_ZONE_BEGIN:
+	case CTL_ZONE_COMMIT:
+	case CTL_ZONE_ABORT:
+		if (data_type == KNOT_CTL_TYPE_DATA) {
 			printf("%s%s%s%s%s%s%s%s",
 			       (!(*empty)     ? "\n"      : ""),
 			       (error != NULL ? "error: " : ""),
@@ -198,10 +231,10 @@ static void format_data(ctl_cmd_t cmd, knot_ctl_type_t type, knot_ctl_data_t *da
 			       (error != NULL ? ")"       : ""));
 			*empty = false;
 		}
-		if (param != NULL) {
+		if (cmd == CTL_ZONE_STATUS && type != NULL) {
 			printf("%s %s: %s",
-			       (type != KNOT_CTL_TYPE_DATA ? " |" : ""),
-			       param, value);
+			       (data_type != KNOT_CTL_TYPE_DATA ? " |" : ""),
+			       type, value);
 		}
 		break;
 	case CTL_CONF_LIST:
@@ -210,13 +243,13 @@ static void format_data(ctl_cmd_t cmd, knot_ctl_type_t type, knot_ctl_data_t *da
 	case CTL_CONF_GET:
 	case CTL_CONF_SET:
 	case CTL_CONF_UNSET:
-		if (type == KNOT_CTL_TYPE_DATA) {
+		if (data_type == KNOT_CTL_TYPE_DATA) {
 			printf("%s%s%s%s%s%s%s%s%s%s%s%s",
 			       (!(*empty)     ? "\n"       : ""),
 			       (error != NULL ? "error: (" : ""),
 			       (error != NULL ? error      : ""),
 			       (error != NULL ? ") "       : ""),
-			       (param != NULL ? param      : ""),
+			       (sign  != NULL ? sign       : ""),
 			       (key0  != NULL ? key0       : ""),
 			       (id    != NULL ? "["        : ""),
 			       (id    != NULL ? id         : ""),
@@ -230,6 +263,32 @@ static void format_data(ctl_cmd_t cmd, knot_ctl_type_t type, knot_ctl_data_t *da
 			printf(" %s", value);
 		}
 		break;
+	case CTL_ZONE_READ:
+	case CTL_ZONE_DIFF:
+	case CTL_ZONE_GET:
+	case CTL_ZONE_SET:
+	case CTL_ZONE_UNSET:
+		if (data_type == KNOT_CTL_TYPE_DATA) {
+			printf("%s%s%s%s%s%s%s%s%s%s%s%s%s",
+			       (!(*empty)     ? "\n"       : ""),
+			       (error != NULL ? "error: (" : ""),
+			       (error != NULL ? error      : ""),
+			       (error != NULL ? ") "       : ""),
+			       (zone  != NULL ? "["        : ""),
+			       (zone  != NULL ? zone       : ""),
+			       (zone  != NULL ? "] "       : ""),
+			       (sign  != NULL ? sign       : ""),
+			       (owner != NULL ? owner      : ""),
+			       (ttl   != NULL ? " "        : ""),
+			       (ttl   != NULL ? ttl        : ""),
+			       (type  != NULL ? " "        : ""),
+			       (type  != NULL ? type       : ""));
+			*empty = false;
+		}
+		if (value != NULL) {
+			printf(" %s", value);
+		}
+		break;
 	default:
 		assert(0);
 	}
@@ -248,8 +307,8 @@ static void format_block(ctl_cmd_t cmd, bool failed, bool empty)
 		printf("%s\n", failed ? "" : "Reloaded");
 		break;
 	case CTL_CONF_BEGIN:
-	case CTL_CONF_ABORT:
 	case CTL_CONF_COMMIT:
+	case CTL_CONF_ABORT:
 	case CTL_CONF_SET:
 	case CTL_CONF_UNSET:
 	case CTL_ZONE_RELOAD:
@@ -257,9 +316,17 @@ static void format_block(ctl_cmd_t cmd, bool failed, bool empty)
 	case CTL_ZONE_RETRANSFER:
 	case CTL_ZONE_FLUSH:
 	case CTL_ZONE_SIGN:
+	case CTL_ZONE_BEGIN:
+	case CTL_ZONE_COMMIT:
+	case CTL_ZONE_ABORT:
+	case CTL_ZONE_SET:
+	case CTL_ZONE_UNSET:
 		printf("%s\n", failed ? "" : "OK");
 		break;
 	case CTL_ZONE_STATUS:
+	case CTL_ZONE_READ:
+	case CTL_ZONE_DIFF:
+	case CTL_ZONE_GET:
 	case CTL_CONF_LIST:
 	case CTL_CONF_READ:
 	case CTL_CONF_DIFF:
@@ -282,13 +349,13 @@ static int ctl_receive(cmd_args_t *args)
 
 		int ret = knot_ctl_receive(args->ctl, &type, &data);
 		if (ret != KNOT_EOK) {
-			log_error("failed to control (%s)", knot_strerror(ret));
+			log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 			return ret;
 		}
 
 		switch (type) {
 		case KNOT_CTL_TYPE_END:
-			log_error("failed to control (%s)", knot_strerror(KNOT_EMALF));
+			log_error(CTL_LOG_STR" (%s)", knot_strerror(KNOT_EMALF));
 			return KNOT_EMALF;
 		case KNOT_CTL_TYPE_BLOCK:
 			format_block(args->desc->cmd, failed, empty);
@@ -299,6 +366,7 @@ static int ctl_receive(cmd_args_t *args)
 			break;
 		default:
 			assert(0);
+			return KNOT_EINVAL;
 		}
 
 		if (data[KNOT_CTL_IDX_ERROR] != NULL) {
@@ -311,26 +379,27 @@ static int ctl_receive(cmd_args_t *args)
 
 static int cmd_ctl(cmd_args_t *args)
 {
-	int ret = check_args(args, 0);
+	int ret = check_args(args, 0, 0);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
 	knot_ctl_data_t data = {
-		[KNOT_CTL_IDX_CMD] = ctl_cmd_to_str(args->desc->cmd)
+		[KNOT_CTL_IDX_CMD] = ctl_cmd_to_str(args->desc->cmd),
+		[KNOT_CTL_IDX_FLAGS] = args->force ? CTL_FLAG_FORCE : NULL
 	};
 
 	// Send the command.
 	ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_DATA, &data);
 	if (ret != KNOT_EOK) {
-		log_error("failed to control (%s)", knot_strerror(ret));
+		log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 		return ret;
 	}
 
 	// Finish the input block.
 	ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_BLOCK, NULL);
 	if (ret != KNOT_EOK) {
-		log_error("failed to control (%s)", knot_strerror(ret));
+		log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 		return ret;
 	}
 
@@ -480,13 +549,25 @@ static int cmd_zone_memstats(cmd_args_t *args)
 static int cmd_zone_ctl(cmd_args_t *args)
 {
 	knot_ctl_data_t data = {
-		[KNOT_CTL_IDX_CMD] = ctl_cmd_to_str(args->desc->cmd)
+		[KNOT_CTL_IDX_CMD] = ctl_cmd_to_str(args->desc->cmd),
+		[KNOT_CTL_IDX_FLAGS] = args->force ? CTL_FLAG_FORCE : NULL
 	};
 
+	// Check the number of arguments.
+	int ret = check_args(args, (args->desc->flags & CMD_FREQ_ZONE) ? 1 : 0, -1);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	// Ignore all zones argument.
+	if (args->argc == 1 && strcmp(args->argv[0], "--") == 0) {
+		args->argc = 0;
+	}
+
 	if (args->argc == 0) {
 		int ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_DATA, &data);
 		if (ret != KNOT_EOK) {
-			log_error("failed to control (%s)", knot_strerror(ret));
+			log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 			return ret;
 		}
 	}
@@ -495,15 +576,131 @@ static int cmd_zone_ctl(cmd_args_t *args)
 
 		int ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_DATA, &data);
 		if (ret != KNOT_EOK) {
-			log_error("failed to control (%s)", knot_strerror(ret));
+			log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 			return ret;
 		}
 	}
 
 	// Finish the input block.
-	int ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_BLOCK, NULL);
+	ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_BLOCK, NULL);
 	if (ret != KNOT_EOK) {
-		log_error("failed to control (%s)", knot_strerror(ret));
+		log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
+		return ret;
+	}
+
+	return ctl_receive(args);
+}
+
+static int set_rdata(cmd_args_t *args, int pos, char *rdata, size_t rdata_len)
+{
+	rdata[0] = '\0';
+
+	for (int i = pos; i < args->argc; i++) {
+		if (i > pos && strlcat(rdata, " ", rdata_len) >= rdata_len) {
+			return KNOT_ESPACE;
+		}
+		if (strlcat(rdata, args->argv[i], rdata_len) >= rdata_len) {
+			return KNOT_ESPACE;
+		}
+	}
+
+	return KNOT_EOK;
+}
+
+static int set_node_items(cmd_args_t *args, knot_ctl_data_t *data, char *rdata,
+                          size_t rdata_len)
+{
+	int min_args, max_args;
+	switch (args->desc->cmd) {
+	case CTL_ZONE_READ:
+	case CTL_ZONE_GET:   min_args = 1; max_args =  3; break;
+	case CTL_ZONE_DIFF:  min_args = 1; max_args =  1; break;
+	case CTL_ZONE_SET:   min_args = 3; max_args = -1; break;
+	case CTL_ZONE_UNSET: min_args = 2; max_args = -1; break;
+	default:
+		assert(0);
+		return KNOT_EINVAL;
+	}
+
+	// Check the number of arguments.
+	int ret = check_args(args, min_args, max_args);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	int idx = 0;
+
+	// Set ZONE name.
+	assert(args->argc > idx);
+	if (strcmp(args->argv[idx], "--") != 0) {
+		(*data)[KNOT_CTL_IDX_ZONE] = args->argv[idx];
+	}
+	idx++;
+
+	// Set OWNER name if specified.
+	if (args->argc > idx) {
+		(*data)[KNOT_CTL_IDX_OWNER] = args->argv[idx];
+		idx++;
+	}
+
+	// Set TTL only with an editing operation.
+	if (args->argc > idx) {
+		uint16_t type;
+		if (knot_rrtype_from_string(args->argv[idx], &type) != 0) {
+			switch (args->desc->cmd) {
+			case CTL_ZONE_SET:
+			case CTL_ZONE_UNSET:
+				(*data)[KNOT_CTL_IDX_TTL] = args->argv[idx];
+				idx++;
+				break;
+			default:
+				return KNOT_EINVAL;
+			}
+		}
+	}
+
+	// Set record TYPE if specified.
+	if (args->argc > idx) {
+		(*data)[KNOT_CTL_IDX_TYPE] = args->argv[idx];
+		idx++;
+	}
+
+	// Set record DATA if specified.
+	if (args->argc > idx) {
+		ret = set_rdata(args, idx, rdata, rdata_len);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+		(*data)[KNOT_CTL_IDX_DATA] = rdata;
+	}
+
+	return KNOT_EOK;
+}
+
+static int cmd_zone_node_ctl(cmd_args_t *args)
+{
+	knot_ctl_data_t data = {
+		[KNOT_CTL_IDX_CMD] = ctl_cmd_to_str(args->desc->cmd),
+		[KNOT_CTL_IDX_FLAGS] = args->force ? CTL_FLAG_FORCE : NULL
+	};
+
+	char rdata[65536]; // Maximum item size in libknot control interface.
+
+	int ret = set_node_items(args, &data, rdata, sizeof(rdata));
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_DATA, &data);
+	if (ret != KNOT_EOK) {
+		log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
+		return ret;
+	}
+
+	// Finish the input block.
+	ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_BLOCK, NULL);
+	if (ret != KNOT_EOK) {
+		log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 		return ret;
 	}
 
@@ -512,14 +709,14 @@ static int cmd_zone_ctl(cmd_args_t *args)
 
 static int cmd_conf_init(cmd_args_t *args)
 {
-	int ret = check_args(args, 0);
+	int ret = check_args(args, 0, 0);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
 	ret = conf_db_check(conf(), &conf()->read_txn);
 	if ((ret >= KNOT_EOK || ret == KNOT_CONF_EVERSION)) {
-		if (ret != KNOT_EOK && !(args->flags & CMD_FFORCE)) {
+		if (ret != KNOT_EOK && !args->force) {
 			log_error("use force option to overwrite the existing "
 			          "destination and ensure the server is not running!");
 			return KNOT_EDENIED;
@@ -539,7 +736,7 @@ static int cmd_conf_init(cmd_args_t *args)
 
 static int cmd_conf_check(cmd_args_t *args)
 {
-	int ret = check_args(args, 0);
+	int ret = check_args(args, 0, 0);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -551,14 +748,14 @@ static int cmd_conf_check(cmd_args_t *args)
 
 static int cmd_conf_import(cmd_args_t *args)
 {
-	int ret = check_args(args, 1);
+	int ret = check_args(args, 1, 1);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
 
 	ret = conf_db_check(conf(), &conf()->read_txn);
 	if ((ret >= KNOT_EOK || ret == KNOT_CONF_EVERSION)) {
-		if (ret != KNOT_EOK && !(args->flags & CMD_FFORCE)) {
+		if (ret != KNOT_EOK && !args->force) {
 			log_error("use force option to overwrite the existing "
 			          "destination and ensure the server is not running!");
 			return KNOT_EDENIED;
@@ -580,7 +777,7 @@ static int cmd_conf_import(cmd_args_t *args)
 
 static int cmd_conf_export(cmd_args_t *args)
 {
-	int ret = check_args(args, 1);
+	int ret = check_args(args, 1, 1);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
@@ -607,14 +804,15 @@ static int cmd_conf_ctl(cmd_args_t *args)
 	}
 
 	knot_ctl_data_t data = {
-		[KNOT_CTL_IDX_CMD] = ctl_cmd_to_str(args->desc->cmd)
+		[KNOT_CTL_IDX_CMD] = ctl_cmd_to_str(args->desc->cmd),
+		[KNOT_CTL_IDX_FLAGS] = args->force ? CTL_FLAG_FORCE : NULL
 	};
 
 	// Send the command without parameters.
 	if (args->argc == 0) {
 		ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_DATA, &data);
 		if (ret != KNOT_EOK) {
-			log_error("failed to control (%s)", knot_strerror(ret));
+			log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 			return ret;
 		}
 	// Set the first item argument.
@@ -625,10 +823,10 @@ static int cmd_conf_ctl(cmd_args_t *args)
 		}
 
 		// Send if only one argument or item without values.
-		if (args->argc == 1 || !(args->desc->flags & CMD_CONF_FOPT_DATA)) {
+		if (args->argc == 1 || !(args->desc->flags & CMD_FOPT_DATA)) {
 			ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_DATA, &data);
 			if (ret != KNOT_EOK) {
-				log_error("failed to control (%s)", knot_strerror(ret));
+				log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 				return ret;
 			}
 		}
@@ -636,7 +834,7 @@ static int cmd_conf_ctl(cmd_args_t *args)
 
 	// Send the item values or the other items.
 	for (int i = 1; i < args->argc; i++) {
-		if (args->desc->flags & CMD_CONF_FOPT_DATA) {
+		if (args->desc->flags & CMD_FOPT_DATA) {
 			data[KNOT_CTL_IDX_DATA] = args->argv[i];
 		} else {
 			ret = get_conf_key(args->argv[i], &data);
@@ -647,7 +845,7 @@ static int cmd_conf_ctl(cmd_args_t *args)
 
 		ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_DATA, &data);
 		if (ret != KNOT_EOK) {
-			log_error("failed to control (%s)", knot_strerror(ret));
+			log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 			return ret;
 		}
 	}
@@ -655,7 +853,7 @@ static int cmd_conf_ctl(cmd_args_t *args)
 	// Finish the input block.
 	ret = knot_ctl_send(args->ctl, KNOT_CTL_TYPE_BLOCK, NULL);
 	if (ret != KNOT_EOK) {
-		log_error("failed to control (%s)", knot_strerror(ret));
+		log_error(CTL_LOG_STR" (%s)", knot_strerror(ret));
 		return ret;
 	}
 
@@ -669,60 +867,78 @@ const cmd_desc_t cmd_table[] = {
 	{ CMD_STOP,            cmd_ctl,           CTL_STOP },
 	{ CMD_RELOAD,          cmd_ctl,           CTL_RELOAD },
 
-	{ CMD_ZONE_CHECK,      cmd_zone_check,    CTL_NONE,            CMD_CONF_FOPT_ZONE | CMD_CONF_FREAD },
-	{ CMD_ZONE_MEMSTATS,   cmd_zone_memstats, CTL_NONE,            CMD_CONF_FOPT_ZONE | CMD_CONF_FREAD },
-	{ CMD_ZONE_STATUS,     cmd_zone_ctl,      CTL_ZONE_STATUS,     CMD_CONF_FOPT_ZONE },
-	{ CMD_ZONE_RELOAD,     cmd_zone_ctl,      CTL_ZONE_RELOAD,     CMD_CONF_FOPT_ZONE },
-	{ CMD_ZONE_REFRESH,    cmd_zone_ctl,      CTL_ZONE_REFRESH,    CMD_CONF_FOPT_ZONE },
-	{ CMD_ZONE_RETRANSFER, cmd_zone_ctl,      CTL_ZONE_RETRANSFER, CMD_CONF_FOPT_ZONE },
-	{ CMD_ZONE_FLUSH,      cmd_zone_ctl,      CTL_ZONE_FLUSH,      CMD_CONF_FOPT_ZONE },
-	{ CMD_ZONE_SIGN,       cmd_zone_ctl,      CTL_ZONE_SIGN,       CMD_CONF_FOPT_ZONE },
+	{ CMD_ZONE_CHECK,      cmd_zone_check,    CTL_NONE,            CMD_FOPT_ZONE | CMD_FREAD },
+	{ CMD_ZONE_MEMSTATS,   cmd_zone_memstats, CTL_NONE,            CMD_FOPT_ZONE | CMD_FREAD },
+	{ CMD_ZONE_STATUS,     cmd_zone_ctl,      CTL_ZONE_STATUS,     CMD_FOPT_ZONE },
+	{ CMD_ZONE_RELOAD,     cmd_zone_ctl,      CTL_ZONE_RELOAD,     CMD_FOPT_ZONE },
+	{ CMD_ZONE_REFRESH,    cmd_zone_ctl,      CTL_ZONE_REFRESH,    CMD_FOPT_ZONE },
+	{ CMD_ZONE_RETRANSFER, cmd_zone_ctl,      CTL_ZONE_RETRANSFER, CMD_FOPT_ZONE },
+	{ CMD_ZONE_FLUSH,      cmd_zone_ctl,      CTL_ZONE_FLUSH,      CMD_FOPT_ZONE },
+	{ CMD_ZONE_SIGN,       cmd_zone_ctl,      CTL_ZONE_SIGN,       CMD_FOPT_ZONE },
 
-	{ CMD_CONF_INIT,       cmd_conf_init,     CTL_NONE,            CMD_CONF_FWRITE },
-	{ CMD_CONF_CHECK,      cmd_conf_check,    CTL_NONE,            CMD_CONF_FREAD },
-	{ CMD_CONF_IMPORT,     cmd_conf_import,   CTL_NONE,            CMD_CONF_FWRITE },
-	{ CMD_CONF_EXPORT,     cmd_conf_export,   CTL_NONE,            CMD_CONF_FREAD },
-	{ CMD_CONF_LIST,       cmd_conf_ctl,      CTL_CONF_LIST,       CMD_CONF_FOPT_ITEM },
-	{ CMD_CONF_READ,       cmd_conf_ctl,      CTL_CONF_READ,       CMD_CONF_FOPT_ITEM },
+	{ CMD_ZONE_READ,       cmd_zone_node_ctl, CTL_ZONE_READ,       CMD_FREQ_ZONE },
+	{ CMD_ZONE_BEGIN,      cmd_zone_ctl,      CTL_ZONE_BEGIN,      CMD_FREQ_ZONE | CMD_FOPT_ZONE },
+	{ CMD_ZONE_COMMIT,     cmd_zone_ctl,      CTL_ZONE_COMMIT,     CMD_FREQ_ZONE | CMD_FOPT_ZONE },
+	{ CMD_ZONE_ABORT,      cmd_zone_ctl,      CTL_ZONE_ABORT,      CMD_FREQ_ZONE | CMD_FOPT_ZONE },
+	{ CMD_ZONE_DIFF,       cmd_zone_node_ctl, CTL_ZONE_DIFF,       CMD_FREQ_ZONE },
+	{ CMD_ZONE_GET,        cmd_zone_node_ctl, CTL_ZONE_GET,        CMD_FREQ_ZONE },
+	{ CMD_ZONE_SET,        cmd_zone_node_ctl, CTL_ZONE_SET,        CMD_FREQ_ZONE },
+	{ CMD_ZONE_UNSET,      cmd_zone_node_ctl, CTL_ZONE_UNSET,      CMD_FREQ_ZONE },
+
+	{ CMD_CONF_INIT,       cmd_conf_init,     CTL_NONE,            CMD_FWRITE },
+	{ CMD_CONF_CHECK,      cmd_conf_check,    CTL_NONE,            CMD_FREAD },
+	{ CMD_CONF_IMPORT,     cmd_conf_import,   CTL_NONE,            CMD_FWRITE },
+	{ CMD_CONF_EXPORT,     cmd_conf_export,   CTL_NONE,            CMD_FREAD },
+	{ CMD_CONF_LIST,       cmd_conf_ctl,      CTL_CONF_LIST,       CMD_FOPT_ITEM },
+	{ CMD_CONF_READ,       cmd_conf_ctl,      CTL_CONF_READ,       CMD_FOPT_ITEM },
 	{ CMD_CONF_BEGIN,      cmd_conf_ctl,      CTL_CONF_BEGIN },
 	{ CMD_CONF_COMMIT,     cmd_conf_ctl,      CTL_CONF_COMMIT },
 	{ CMD_CONF_ABORT,      cmd_conf_ctl,      CTL_CONF_ABORT },
-	{ CMD_CONF_DIFF,       cmd_conf_ctl,      CTL_CONF_DIFF,       CMD_CONF_FOPT_ITEM | CMD_CONF_FREQ_TXN },
-	{ CMD_CONF_GET,        cmd_conf_ctl,      CTL_CONF_GET,        CMD_CONF_FOPT_ITEM | CMD_CONF_FREQ_TXN },
-	{ CMD_CONF_SET,        cmd_conf_ctl,      CTL_CONF_SET,        CMD_CONF_FREQ_ITEM | CMD_CONF_FOPT_DATA | CMD_CONF_FREQ_TXN },
-	{ CMD_CONF_UNSET,      cmd_conf_ctl,      CTL_CONF_UNSET,      CMD_CONF_FOPT_ITEM | CMD_CONF_FOPT_DATA | CMD_CONF_FREQ_TXN },
+	{ CMD_CONF_DIFF,       cmd_conf_ctl,      CTL_CONF_DIFF,       CMD_FOPT_ITEM | CMD_FREQ_TXN },
+	{ CMD_CONF_GET,        cmd_conf_ctl,      CTL_CONF_GET,        CMD_FOPT_ITEM | CMD_FREQ_TXN },
+	{ CMD_CONF_SET,        cmd_conf_ctl,      CTL_CONF_SET,        CMD_FREQ_ITEM | CMD_FOPT_DATA | CMD_FREQ_TXN },
+	{ CMD_CONF_UNSET,      cmd_conf_ctl,      CTL_CONF_UNSET,      CMD_FOPT_ITEM | CMD_FOPT_DATA | CMD_FREQ_TXN },
 	{ NULL }
 };
 
 static const cmd_help_t cmd_help_table[] = {
-	{ CMD_EXIT,            "",                     "Exit interactive mode." },
-	{ "",                  "",                     "" },
-	{ CMD_STATUS,          "",                     "Check if the server is running." },
-	{ CMD_STOP,            "",                     "Stop the server if running." },
-	{ CMD_RELOAD,          "",                     "Reload the server configuration and modified zones." },
-	{ "",                  "",                     "" },
-	{ CMD_ZONE_CHECK,      "[<zone>...]",          "Check if the zone can be loaded. (*)" },
-	{ CMD_ZONE_MEMSTATS,   "[<zone>...]",          "Estimate memory use for the zone. (*)" },
-	{ CMD_ZONE_STATUS,     "[<zone>...]",          "Show the zone status." },
-	{ CMD_ZONE_RELOAD,     "[<zone>...]",          "Reload a zone from a disk." },
-	{ CMD_ZONE_REFRESH,    "[<zone>...]",          "Force slave zone refresh." },
-	{ CMD_ZONE_RETRANSFER, "[<zone>...]",          "Force slave zone retransfer (no serial check)." },
-	{ CMD_ZONE_FLUSH,      "[<zone>...]",          "Flush zone journal into the zone file." },
-	{ CMD_ZONE_SIGN,       "[<zone>...]",          "Re-sign the automatically signed zone." },
-	{ "",                  "",                     "" },
-	{ CMD_CONF_INIT,       "",                     "Initialize the confdb. (*)" },
-	{ CMD_CONF_CHECK,      "",                     "Check the server configuration. (*)" },
-	{ CMD_CONF_IMPORT,     "<filename>",           "Import a config file into the confdb. (*)" },
-	{ CMD_CONF_EXPORT,     "<filename>",           "Export the confdb into a config file. (*)" },
-	{ CMD_CONF_LIST,       "[<item>...]",          "List the confdb sections or section items." },
-	{ CMD_CONF_READ,       "[<item>...]",          "Read the item from the active confdb." },
-	{ CMD_CONF_BEGIN,      "",                     "Begin a writing confdb transaction." },
-	{ CMD_CONF_COMMIT,     "",                     "Commit the confdb transaction." },
-	{ CMD_CONF_ABORT,      "",                     "Rollback the confdb transaction." },
-	{ CMD_CONF_DIFF,       "[<item>...]",          "Get the item difference in the transaction." },
-	{ CMD_CONF_GET,        "[<item>...]",          "Get the item data from the transaction." },
-	{ CMD_CONF_SET,        " <item>  [<data>...]", "Set the item data in the transaction." },
-	{ CMD_CONF_UNSET,      "[<item>] [<data>...]", "Unset the item data in the transaction." },
+	{ CMD_EXIT,            "",                                       "Exit interactive mode." },
+	{ "",                  "",                                       "" },
+	{ CMD_STATUS,          "",                                       "Check if the server is running." },
+	{ CMD_STOP,            "",                                       "Stop the server if running." },
+	{ CMD_RELOAD,          "",                                       "Reload the server configuration and modified zones." },
+	{ "",                  "",                                       "" },
+	{ CMD_ZONE_CHECK,      "[<zone>...]",                            "Check if the zone can be loaded. (*)" },
+	{ CMD_ZONE_MEMSTATS,   "[<zone>...]",                            "Estimate memory use for the zone. (*)" },
+	{ CMD_ZONE_STATUS,     "[<zone>...]",                            "Show the zone status." },
+	{ CMD_ZONE_RELOAD,     "[<zone>...]",                            "Reload a zone from a disk." },
+	{ CMD_ZONE_REFRESH,    "[<zone>...]",                            "Force slave zone refresh." },
+	{ CMD_ZONE_RETRANSFER, "[<zone>...]",                            "Force slave zone retransfer (no serial check)." },
+	{ CMD_ZONE_FLUSH,      "[<zone>...]",                            "Flush zone journal into the zone file." },
+	{ CMD_ZONE_SIGN,       "[<zone>...]",                            "Re-sign the automatically signed zone." },
+	{ "",                  "",                                       "" },
+	{ CMD_ZONE_READ,       "<zone> [<owner> [<type>]]",              "Get zone data that are currently being presented." },
+	{ CMD_ZONE_BEGIN,      "<zone>...",                              "Begin a zone transaction." },
+	{ CMD_ZONE_COMMIT,     "<zone>...",                              "Commit the zone transaction." },
+	{ CMD_ZONE_ABORT,      "<zone>...",                              "Abort the zone transaction." },
+	{ CMD_ZONE_DIFF,       "<zone>",                                 "Get zone changes within the transaction." },
+	{ CMD_ZONE_GET,        "<zone> [<owner> [<type>]]",              "Get zone data within the transaction." },
+	{ CMD_ZONE_SET,        "<zone>  <owner> [<ttl>] <type> <rdata>", "Add zone record within the transaction." },
+	{ CMD_ZONE_UNSET,      "<zone>  <owner> [<type> [<rdata>]]",     "Remove zone data within the transaction." },
+	{ "",                  "",                                       "" },
+	{ CMD_CONF_INIT,       "",                                       "Initialize the confdb. (*)" },
+	{ CMD_CONF_CHECK,      "",                                       "Check the server configuration. (*)" },
+	{ CMD_CONF_IMPORT,     "<filename>",                             "Import a config file into the confdb. (*)" },
+	{ CMD_CONF_EXPORT,     "<filename>",                             "Export the confdb into a config file. (*)" },
+	{ CMD_CONF_LIST,       "[<item>...]",                            "List the confdb sections or section items." },
+	{ CMD_CONF_READ,       "[<item>...]",                            "Get the item from the active confdb." },
+	{ CMD_CONF_BEGIN,      "",                                       "Begin a writing confdb transaction." },
+	{ CMD_CONF_COMMIT,     "",                                       "Commit the confdb transaction." },
+	{ CMD_CONF_ABORT,      "",                                       "Rollback the confdb transaction." },
+	{ CMD_CONF_DIFF,       "[<item>...]",                            "Get the item difference within the transaction." },
+	{ CMD_CONF_GET,        "[<item>...]",                            "Get the item data within the transaction." },
+	{ CMD_CONF_SET,        " <item>  [<data>...]",                   "Set the item data within the transaction." },
+	{ CMD_CONF_UNSET,      "[<item>] [<data>...]",                   "Unset the item data within the transaction." },
 	{ NULL }
 };
 
@@ -731,12 +947,13 @@ void print_commands(void)
 	printf("\nActions:\n");
 
 	for (const cmd_help_t *cmd = cmd_help_table; cmd->name != NULL; cmd++) {
-		printf(" %-15s %-20s %s\n", cmd->name, cmd->params, cmd->desc);
+		printf(" %-15s %-38s %s\n", cmd->name, cmd->params, cmd->desc);
 	}
 
 	printf("\n"
 	       "Note:\n"
-	       " Empty <zone> parameter means all zones.\n"
+	       " Use @ owner to denote the zone name.\n"
+	       " Empty or '--' <zone> parameter means all zones or all zones with a transaction.\n"
 	       " Type <item> parameter in the form of <section>[<identifier>].<name>.\n"
 	       " (*) indicates a local operation which requires a configuration.\n");
 }
