@@ -49,20 +49,20 @@ int main(int argc, char *argv[])
 	uint8_t data[8] = "\7teststr";
 	knot_rrset_add_rdata(apex_txt_rr, data, sizeof(data), 3600, NULL);
 
-	int ret = changeset_add_rrset(ch, apex_txt_rr);
+	int ret = changeset_add_rrset(ch, apex_txt_rr, CHANGESET_CHECK);
 	ok(ret == KNOT_EOK, "changeset: add RRSet");
 	ok(changeset_size(ch) == 1, "changeset: size add");
-	ret = changeset_rem_rrset(ch, apex_txt_rr);
+	ret = changeset_rem_rrset(ch, apex_txt_rr, CHANGESET_CHECK);
 	ok(ret == KNOT_EOK, "changeset: rem RRSet");
-	ok(changeset_size(ch) == 2, "changeset: size remove");
-
-	ok(!changeset_empty(ch), "changeset: not empty");
+	ok(changeset_size(ch) == 1, "changeset: size remove");
+	ok(!changeset_empty(ch), "changeset: empty");
+	changeset_add_rrset(ch, apex_txt_rr, CHANGESET_CHECK);
 
 	// Add another RR to node.
 	knot_rrset_t *apex_spf_rr = knot_rrset_new(d, KNOT_RRTYPE_SPF, KNOT_CLASS_IN, NULL);
 	assert(apex_spf_rr);
 	knot_rrset_add_rdata(apex_spf_rr, data, sizeof(data), 3600, NULL);
-	ret = changeset_add_rrset(ch, apex_spf_rr);
+	ret = changeset_add_rrset(ch, apex_spf_rr, CHANGESET_CHECK);
 	ok(ret == KNOT_EOK, "changeset: add multiple");
 
 	// Add another node.
@@ -72,7 +72,7 @@ int main(int argc, char *argv[])
 	knot_rrset_t *other_rr = knot_rrset_new(d, KNOT_RRTYPE_TXT, KNOT_CLASS_IN, NULL);
 	assert(other_rr);
 	knot_rrset_add_rdata(other_rr, data, sizeof(data), 3600, NULL);
-	ret = changeset_add_rrset(ch, other_rr);
+	ret = changeset_add_rrset(ch, other_rr, CHANGESET_CHECK);
 	ok(ret == KNOT_EOK, "changeset: remove multiple");
 
 	// Test add traversal.
@@ -93,6 +93,9 @@ int main(int argc, char *argv[])
 	changeset_iter_clear(&it);
 	ok(knot_rrset_empty(&iter), "changeset: traversal: skip non-terminals");
 
+	changeset_rem_rrset(ch, apex_txt_rr, CHANGESET_CHECK);
+	changeset_rem_rrset(ch, apex_txt_rr, CHANGESET_CHECK);
+
 	// Test remove traversal.
 	ret = changeset_iter_rem(&it, ch, false);
 	ok(ret == KNOT_EOK, "changeset: create iter rem");
@@ -111,7 +114,7 @@ int main(int argc, char *argv[])
 		iter = changeset_iter_next(&it);
 	}
 	changeset_iter_clear(&it);
-	ok(size == 4, "changeset: iter all");
+	ok(size == 3, "changeset: iter all");
 
 	// Create new changeset.
 	knot_dname_free(&d, NULL);
@@ -124,7 +127,7 @@ int main(int argc, char *argv[])
 	knot_dname_free(&apex_txt_rr->owner, NULL);
 	apex_txt_rr->owner = knot_dname_from_str_alloc("something.test.");
 	assert(apex_txt_rr->owner);
-	ret = changeset_add_rrset(ch2, apex_txt_rr);
+	ret = changeset_add_rrset(ch2, apex_txt_rr, CHANGESET_CHECK);
 	assert(ret == KNOT_EOK);
 
 	// Add something to remove section.
@@ -132,12 +135,12 @@ int main(int argc, char *argv[])
 	apex_txt_rr->owner =
 		knot_dname_from_str_alloc("and.now.for.something.completely.different.test.");
 	assert(apex_txt_rr->owner);
-	ret = changeset_rem_rrset(ch2, apex_txt_rr);
+	ret = changeset_rem_rrset(ch2, apex_txt_rr, CHANGESET_CHECK);
 	assert(ret == KNOT_EOK);
 
 	// Test merge.
 	ret = changeset_merge(ch, ch2);
-	ok(ret == KNOT_EOK && changeset_size(ch) == 6, "changeset: merge");
+	ok(ret == KNOT_EOK && changeset_size(ch) == 5, "changeset: merge");
 
 	// Test cleanup.
 	changeset_clear(ch);
